@@ -38,7 +38,6 @@
 #include <libempathy/empathy-individual-manager.h>
 #include <libempathy/empathy-chatroom-manager.h>
 #include <libempathy/empathy-utils.h>
-#include <libempathy/empathy-contact-list.h>
 #include <libempathy/empathy-pkg-kit.h>
 
 #include "empathy-account-selector-dialog.h"
@@ -1560,8 +1559,7 @@ room_sub_menu_activate_cb (GtkWidget *item,
   g_return_if_fail (EMPATHY_IS_CONTACT (contact));
 
   /* send invitation */
-  empathy_contact_list_add (EMPATHY_CONTACT_LIST (chat),
-      contact, _("Inviting you to this room"));
+  empathy_tp_chat_add (chat, contact, _("Inviting you to this room"));
 
 out:
   g_object_unref (contact);
