@@ -30,10 +30,8 @@
 #include <telepathy-glib/simple-approver.h>
 
 #include <libempathy/empathy-presence-manager.h>
-#include <libempathy/empathy-tp-contact-factory.h>
 #include <libempathy/empathy-connection-aggregator.h>
 #include <libempathy/empathy-tp-chat.h>
-#include <libempathy/empathy-tp-streamed-media.h>
 #include <libempathy/empathy-utils.h>
 #include <libempathy/empathy-gsettings.h>
 
@@ -408,11 +406,7 @@ reject_channel_claim_cb (GObject *source,
       goto out;
     }
 
-  if (EMPATHY_IS_TP_STREAMED_MEDIA (user_data))
-    {
-      empathy_tp_streamed_media_close (user_data);
-    }
-  else if (TP_IS_CALL_CHANNEL (user_data))
+  if (TP_IS_CALL_CHANNEL (user_data))
     {
       tp_call_channel_hangup_async (user_data,
           TP_CALL_STATE_CHANGE_REASON_USER_REQUESTED,
@@ -499,13 +493,7 @@ event_channel_process_voip_func (EventPriv *event)
       return;
     }
 
-  if (etype == EMPATHY_EVENT_TYPE_VOIP)
-    {
-      EmpathyTpStreamedMedia *call;
-      call = EMPATHY_TP_STREAMED_MEDIA (event->approval->handler_instance);
-      video = empathy_tp_streamed_media_has_initial_video (call);
-    }
-  else if (etype == EMPATHY_EVENT_TYPE_CALL)
+  if (etype == EMPATHY_EVENT_TYPE_CALL)
     {
       TpCallChannel *call;
       call = TP_CALL_CHANNEL (event->approval->handler_instance);
@@ -588,7 +576,6 @@ event_manager_chat_message_received_cb (EmpathyTpChat *tp_chat,
   EmpathyMessage *message,
   EventManagerApproval *approval)
 {
-  GtkWidget       *window;
   EmpathyContact  *sender;
   const gchar     *header;
   const gchar     *msg;
@@ -616,12 +603,8 @@ event_manager_chat_message_received_cb (EmpathyTpChat *tp_chat,
         EMPATHY_EVENT_TYPE_CHAT, EMPATHY_IMAGE_NEW_MESSAGE, header, msg,
         approval, event_text_channel_process_func, NULL);
 
-  window = empathy_roster_window_dup ();
-
-  empathy_sound_manager_play (priv->sound_mgr, window,
+  empathy_sound_manager_play (priv->sound_mgr, NULL,
       EMPATHY_SOUND_CONVERSATION_NEW);
-
-  g_object_unref (window);
 }
 
 static void
@@ -689,112 +672,6 @@ event_manager_call_state_changed_cb (TpCallChannel *call,
     }
 }
 
-static void
-event_manager_call_channel_got_contact_cb (TpConnection *connection,
-                                 EmpathyContact *contact,
-                                 const GError *error,
-                                 gpointer user_data,
-                                 GObject *object)
-{
-  EventManagerApproval *approval = (EventManagerApproval *) user_data;
-  EmpathyEventManagerPriv *priv = GET_PRIV (approval->manager);
-  GtkWidget *window;
-  TpCallChannel *call;
-  gchar *header;
-  gboolean video;
-
-  call = TP_CALL_CHANNEL (approval->handler_instance);
-
-  if (error != NULL)
-    {
-      DEBUG ("Can't get the contact for the call.. Rejecting?");
-      reject_approval (approval);
-      return;
-    }
-
-  if (tp_call_channel_get_state (call, NULL, NULL, NULL) == TP_CALL_STATE_ENDED)
-    {
-      DEBUG ("Call already ended, seems we missed it :/");
-      reject_approval (approval);
-      return;
-    }
-
-  approval->handler = g_signal_connect (call, "state-changed",
-    G_CALLBACK (event_manager_call_state_changed_cb), approval);
-
-  window = empathy_roster_window_dup ();
-  approval->contact = g_object_ref (contact);
-
-  g_object_get (G_OBJECT (call), "initial-video", &video, NULL);
-
-  header = g_strdup_printf (
-    video ? _("Incoming video call from %s") :_("Incoming call from %s"),
-    empathy_contact_get_alias (approval->contact));
-
-  event_manager_add (approval->manager, NULL,
-      approval->contact, EMPATHY_EVENT_TYPE_CALL,
-      video ? EMPATHY_IMAGE_VIDEO_CALL : EMPATHY_IMAGE_VOIP,
-      header, NULL, approval,
-      event_channel_process_voip_func, NULL);
-
-  g_free (header);
-
-  priv->ringing++;
-  if (priv->ringing == 1)
-    empathy_sound_manager_start_playing (priv->sound_mgr, window,
-        EMPATHY_SOUND_PHONE_INCOMING, MS_BETWEEN_RING);
-
-  g_object_unref (window);
-}
-
-static void
-event_manager_media_channel_got_contact (EventManagerApproval *approval)
-{
-  EmpathyEventManagerPriv *priv = GET_PRIV (approval->manager);
-  GtkWidget *window = empathy_roster_window_dup ();
-  gchar *header;
-  EmpathyTpStreamedMedia *call;
-  gboolean video;
-
-  call = EMPATHY_TP_STREAMED_MEDIA (approval->handler_instance);
-
-  video = empathy_tp_streamed_media_has_initial_video (call);
-
-  header = g_strdup_printf (
-    video ? _("Incoming video call from %s") :_("Incoming call from %s"),
-    empathy_contact_get_alias (approval->contact));
-
-  event_manager_add (approval->manager, NULL,
-      approval->contact, EMPATHY_EVENT_TYPE_VOIP,
-      video ? EMPATHY_IMAGE_VIDEO_CALL : EMPATHY_IMAGE_VOIP,
-      header, NULL, approval,
-      event_channel_process_voip_func, NULL);
-
-  g_free (header);
-
-  priv->ringing++;
-  if (priv->ringing == 1)
-    empathy_sound_manager_start_playing (priv->sound_mgr, window,
-        EMPATHY_SOUND_PHONE_INCOMING, MS_BETWEEN_RING);
-
-  g_object_unref (window);
-}
-
-static void
-event_manager_media_channel_contact_changed_cb (EmpathyTpStreamedMedia *call,
-  GParamSpec *param, EventManagerApproval *approval)
-{
-  EmpathyContact *contact;
-
-  g_object_get (G_OBJECT (call), "contact", &contact, NULL);
-
-  if (contact == NULL)
-    return;
-
-  approval->contact = contact;
-  event_manager_media_channel_got_contact (approval);
-}
-
 static void
 invite_dialog_response_cb (GtkDialog *dialog,
                            gint response,
@@ -871,15 +748,14 @@ event_room_channel_process_func (EventPriv *event)
 static void
 display_invite_room_dialog (EventManagerApproval *approval)
 {
-  GtkWidget *window = empathy_roster_window_dup ();
   const gchar *invite_msg;
   gchar *msg;
-  TpHandle self_handle;
+  TpContact *self_contact;
   EmpathyEventManagerPriv *priv = GET_PRIV (approval->manager);
 
-  self_handle = tp_channel_group_get_self_handle (approval->main_channel);
-  tp_channel_group_get_local_pending_info (approval->main_channel, self_handle,
-      NULL, NULL, &invite_msg);
+  self_contact = tp_channel_group_get_self_contact (approval->main_channel);
+  tp_channel_group_get_local_pending_contact_info (approval->main_channel,
+      self_contact, NULL, NULL, &invite_msg);
 
   if (approval->contact != NULL)
     {
@@ -898,62 +774,10 @@ display_invite_room_dialog (EventManagerApproval *approval)
       EMPATHY_IMAGE_GROUP_MESSAGE, msg, invite_msg, approval,
       event_room_channel_process_func, NULL);
 
-  empathy_sound_manager_play (priv->sound_mgr, window,
+  empathy_sound_manager_play (priv->sound_mgr, NULL,
       EMPATHY_SOUND_CONVERSATION_NEW);
 
   g_free (msg);
-  g_object_unref (window);
-}
-
-static void
-event_manager_muc_invite_got_contact_cb (TpConnection *connection,
-                                         EmpathyContact *contact,
-                                         const GError *error,
-                                         gpointer user_data,
-                                         GObject *object)
-{
-  EventManagerApproval *approval = (EventManagerApproval *) user_data;
-
-  if (error != NULL)
-    {
-      DEBUG ("Error: %s", error->message);
-    }
-  else
-    {
-      approval->contact = g_object_ref (contact);
-    }
-
-  display_invite_room_dialog (approval);
-}
-
-static void
-event_manager_ft_got_contact_cb (TpConnection *connection,
-                                 EmpathyContact *contact,
-                                 const GError *error,
-                                 gpointer user_data,
-                                 GObject *object)
-{
-  EventManagerApproval *approval = (EventManagerApproval *) user_data;
-  GtkWidget *window = empathy_roster_window_dup ();
-  char *header;
-  EmpathyEventManagerPriv *priv = GET_PRIV (approval->manager);
-
-  approval->contact = g_object_ref (contact);
-
-  header = g_strdup_printf (_("Incoming file transfer from %s"),
-                            empathy_contact_get_alias (approval->contact));
-
-  event_manager_add (approval->manager, NULL,
-      approval->contact, EMPATHY_EVENT_TYPE_TRANSFER,
-      EMPATHY_IMAGE_DOCUMENT_SEND, header, NULL,
-      approval, event_channel_process_func, NULL);
-
-  /* FIXME better sound for incoming file transfers ?*/
-  empathy_sound_manager_play (priv->sound_mgr, window,
-      EMPATHY_SOUND_CONVERSATION_NEW);
-
-  g_free (header);
-  g_object_unref (window);
 }
 
 static void
@@ -993,6 +817,184 @@ find_main_channel (GList *channels)
   return text;
 }
 
+static void
+approve_text_channel (EmpathyEventManager *self,
+    EventManagerApproval *approval,
+    TpAddDispatchOperationContext *context,
+    EmpathyTpChat *tp_chat)
+{
+  GList *messages, *l;
+
+  approval->handler_instance = g_object_ref (tp_chat);
+
+  if (tp_proxy_has_interface (tp_chat, TP_IFACE_CHANNEL_INTERFACE_GROUP))
+    {
+      /* Are we in local-pending ? */
+      TpContact *inviter;
+
+      if (empathy_tp_chat_is_invited (tp_chat, &inviter))
+        {
+          /* We are invited to a room */
+          DEBUG ("Have been invited to %s. Ask user if he wants to accept",
+              tp_channel_get_identifier (TP_CHANNEL_GROUP_CHANGE_REASON_NONE));
+
+          if (inviter != NULL)
+            {
+              approval->contact = empathy_contact_dup_from_tp_contact (
+                  inviter);
+            }
+
+          display_invite_room_dialog (approval);
+          tp_add_dispatch_operation_context_accept (context);
+
+          return;
+        }
+
+      /* We are not invited, approve the channel right now */
+      tp_add_dispatch_operation_context_accept (context);
+
+      approval->auto_approved = TRUE;
+      event_manager_approval_approve (approval);
+      return;
+    }
+
+  /* 1-1 text channel, wait for the first message */
+  approval->handler = g_signal_connect (tp_chat, "message-received-empathy",
+    G_CALLBACK (event_manager_chat_message_received_cb), approval);
+
+  messages = (GList *) empathy_tp_chat_get_pending_messages (tp_chat);
+  for (l = messages; l != NULL; l = g_list_next (l))
+    {
+      EmpathyMessage *msg = l->data;
+
+      event_manager_chat_message_received_cb (tp_chat, msg, approval);
+    }
+
+  tp_add_dispatch_operation_context_accept (context);
+}
+
+static void
+approval_set_target_contact (EventManagerApproval *approval,
+    TpChannel *channel)
+{
+  TpContact *contact;
+
+  contact = tp_channel_get_target_contact (channel);
+  approval->contact = empathy_contact_dup_from_tp_contact (contact);
+}
+
+static void
+approve_call_channel (EmpathyEventManager *self,
+    EventManagerApproval *approval,
+    TpAddDispatchOperationContext *context,
+    TpCallChannel *call)
+{
+  TpChannel *channel = TP_CHANNEL (call);
+  EmpathyEventManagerPriv *priv = GET_PRIV (approval->manager);
+  gchar *header;
+  gboolean video;
+
+  approval->handler_instance = g_object_ref (call);
+  approval_set_target_contact (approval, channel);
+
+  tp_add_dispatch_operation_context_accept (context);
+
+  if (tp_call_channel_get_state (call, NULL, NULL, NULL) == TP_CALL_STATE_ENDED)
+    {
+      DEBUG ("Call already ended, seems we missed it :/");
+      reject_approval (approval);
+      return;
+    }
+
+  approval->handler = g_signal_connect (call, "state-changed",
+    G_CALLBACK (event_manager_call_state_changed_cb), approval);
+
+  g_object_get (G_OBJECT (call), "initial-video", &video, NULL);
+
+  header = g_strdup_printf (
+    video ? _("Incoming video call from %s") :_("Incoming call from %s"),
+    empathy_contact_get_alias (approval->contact));
+
+  event_manager_add (approval->manager, NULL,
+      approval->contact, EMPATHY_EVENT_TYPE_CALL,
+      video ? EMPATHY_IMAGE_VIDEO_CALL : EMPATHY_IMAGE_VOIP,
+      header, NULL, approval,
+      event_channel_process_voip_func, NULL);
+
+  g_free (header);
+
+  priv->ringing++;
+  if (priv->ringing == 1)
+    empathy_sound_manager_start_playing (priv->sound_mgr, NULL,
+        EMPATHY_SOUND_PHONE_INCOMING, MS_BETWEEN_RING);
+}
+
+static void
+approve_ft_channel (EmpathyEventManager *self,
+    EventManagerApproval *approval,
+    TpAddDispatchOperationContext *context,
+    TpFileTransferChannel *ft)
+{
+  TpChannel *channel = TP_CHANNEL (ft);
+  EmpathyEventManagerPriv *priv = GET_PRIV (approval->manager);
+  gchar *header;
+
+  approval->handler_instance = g_object_ref (ft);
+  approval_set_target_contact (approval, channel);
+
+  tp_add_dispatch_operation_context_accept (context);
+
+  header = g_strdup_printf (_("Incoming file transfer from %s"),
+      empathy_contact_get_alias (approval->contact));
+
+  event_manager_add (approval->manager, NULL,
+      approval->contact, EMPATHY_EVENT_TYPE_TRANSFER,
+      EMPATHY_IMAGE_DOCUMENT_SEND, header, NULL,
+      approval, event_channel_process_func, NULL);
+
+  /* FIXME better sound for incoming file transfers ?*/
+  empathy_sound_manager_play (priv->sound_mgr, NULL,
+      EMPATHY_SOUND_CONVERSATION_NEW);
+
+  g_free (header);
+}
+
+static void
+approve_sasl_channel (EmpathyEventManager *self,
+    TpAccount *account,
+    EventManagerApproval *approval,
+    TpAddDispatchOperationContext *context,
+    TpChannel *channel)
+{
+  GHashTable *props;
+  const gchar * const *available_mechanisms;
+
+  props = tp_channel_borrow_immutable_properties (channel);
+  available_mechanisms = tp_asv_get_boxed (props,
+      TP_PROP_CHANNEL_INTERFACE_SASL_AUTHENTICATION_AVAILABLE_MECHANISMS,
+      G_TYPE_STRV);
+
+  if (tp_strv_contains (available_mechanisms, "X-TELEPATHY-PASSWORD"))
+    {
+      event_manager_add (approval->manager, account, NULL,
+          EMPATHY_EVENT_TYPE_AUTH,
+          GTK_STOCK_DIALOG_AUTHENTICATION,
+          tp_account_get_display_name (account),
+          _("Password required"), approval,
+          event_manager_auth_process_func, NULL);
+    }
+  else
+    {
+      GError error = { TP_ERROR, TP_ERROR_NOT_IMPLEMENTED,
+          "Support only X-TELEPATHY-PASSWORD auth method" };
+
+      tp_add_dispatch_operation_context_fail (context, &error);
+      return;
+    }
+
+  tp_add_dispatch_operation_context_accept (context);
+}
+
 static void
 approve_channels (TpSimpleApprover *approver,
     TpAccount *account,
@@ -1028,132 +1030,22 @@ approve_channels (TpSimpleApprover *approver,
 
   channel_type = tp_channel_get_channel_type_id (channel);
 
-  if (TP_IS_TEXT_CHANNEL (channel))
+  if (EMPATHY_IS_TP_CHAT (channel))
     {
-      EmpathyTpChat *tp_chat = EMPATHY_TP_CHAT (channel);
-      GList *messages, *l;
-
-      approval->handler_instance = g_object_ref (tp_chat);
-
-      if (tp_proxy_has_interface (channel, TP_IFACE_CHANNEL_INTERFACE_GROUP))
-        {
-          /* Are we in local-pending ? */
-          TpHandle inviter;
-
-          if (empathy_tp_chat_is_invited (tp_chat, &inviter))
-            {
-              /* We are invited to a room */
-              DEBUG ("Have been invited to %s. Ask user if he wants to accept",
-                  tp_channel_get_identifier (channel));
-
-              if (inviter != 0)
-                {
-                  empathy_tp_contact_factory_get_from_handle (connection,
-                      inviter, event_manager_muc_invite_got_contact_cb,
-                      approval, NULL, G_OBJECT (self));
-                }
-              else
-                {
-                  display_invite_room_dialog (approval);
-                }
-
-              goto out;
-            }
-
-          /* We are not invited, approve the channel right now */
-          tp_add_dispatch_operation_context_accept (context);
-
-          approval->auto_approved = TRUE;
-          event_manager_approval_approve (approval);
-          return;
-        }
-
-      /* 1-1 text channel, wait for the first message */
-      approval->handler = g_signal_connect (tp_chat, "message-received-empathy",
-        G_CALLBACK (event_manager_chat_message_received_cb), approval);
-
-      messages = (GList *) empathy_tp_chat_get_pending_messages (tp_chat);
-      for (l = messages; l != NULL; l = g_list_next (l))
-        {
-          EmpathyMessage *msg = l->data;
-
-          event_manager_chat_message_received_cb (tp_chat, msg, approval);
-        }
+      approve_text_channel (self, approval, context, EMPATHY_TP_CHAT (channel));
     }
-  else if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_STREAMED_MEDIA)
+  else if (TP_IS_CALL_CHANNEL (channel))
     {
-      EmpathyContact *contact;
-      EmpathyTpStreamedMedia *call = empathy_tp_streamed_media_new (account,
-        channel);
-
-      approval->handler_instance = G_OBJECT (call);
-
-      g_object_get (G_OBJECT (call), "contact", &contact, NULL);
-
-      if (contact == NULL)
-        {
-          g_signal_connect (call, "notify::contact",
-            G_CALLBACK (event_manager_media_channel_contact_changed_cb),
-            approval);
-        }
-      else
-        {
-          approval->contact = contact;
-          event_manager_media_channel_got_contact (approval);
-        }
-
+      approve_call_channel (self, approval, context, TP_CALL_CHANNEL (channel));
     }
-  else if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_CALL)
+  else if (TP_IS_FILE_TRANSFER_CHANNEL (channel))
     {
-      TpCallChannel *call = TP_CALL_CHANNEL (channel);
-      const gchar *id;
-
-      approval->handler_instance = g_object_ref (call);
-
-      id = tp_channel_get_identifier (channel);
-
-      empathy_tp_contact_factory_get_from_id (connection, id,
-        event_manager_call_channel_got_contact_cb,
-        approval, NULL, G_OBJECT (self));
-    }
-  else if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_FILE_TRANSFER)
-    {
-      TpHandle handle;
-
-      approval->handler_instance = g_object_ref (channel);
-
-      handle = tp_channel_get_handle (channel, NULL);
-
-      empathy_tp_contact_factory_get_from_handle (connection, handle,
-        event_manager_ft_got_contact_cb, approval, NULL, G_OBJECT (self));
+      approve_ft_channel (self, approval, context,
+          TP_FILE_TRANSFER_CHANNEL (channel));
     }
   else if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_SERVER_AUTHENTICATION)
     {
-      GHashTable *props;
-      const gchar * const *available_mechanisms;
-
-      props = tp_channel_borrow_immutable_properties (channel);
-      available_mechanisms = tp_asv_get_boxed (props,
-          TP_PROP_CHANNEL_INTERFACE_SASL_AUTHENTICATION_AVAILABLE_MECHANISMS,
-          G_TYPE_STRV);
-
-      if (tp_strv_contains (available_mechanisms, "X-TELEPATHY-PASSWORD"))
-        {
-          event_manager_add (approval->manager, account, NULL,
-              EMPATHY_EVENT_TYPE_AUTH,
-              GTK_STOCK_DIALOG_AUTHENTICATION,
-              tp_account_get_display_name (account),
-              _("Password required"), approval,
-              event_manager_auth_process_func, NULL);
-        }
-      else
-        {
-          GError error = { TP_ERROR, TP_ERROR_NOT_IMPLEMENTED,
-              "Support only X-TELEPATHY-PASSWORD auth method" };
-
-          tp_add_dispatch_operation_context_fail (context, &error);
-          return;
-        }
+      approve_sasl_channel (self, account, approval, context, channel);
     }
   else
     {
@@ -1166,9 +1058,6 @@ approve_channels (TpSimpleApprover *approver,
       tp_add_dispatch_operation_context_fail (context, &error);
       return;
     }
-
-out:
-  tp_add_dispatch_operation_context_accept (context);
 }
 
 static void
@@ -1251,7 +1140,6 @@ event_manager_presence_changed_cb (EmpathyContact *contact,
   EmpathyEventManagerPriv *priv = GET_PRIV (manager);
   TpAccount *account;
   EmpathyPresenceManager *presence_mgr;
-  GtkWidget *window = empathy_roster_window_dup ();
 
   account = empathy_contact_get_account (contact);
   presence_mgr = empathy_presence_manager_dup_singleton ();
@@ -1267,7 +1155,7 @@ event_manager_presence_changed_cb (EmpathyContact *contact,
           TP_CONNECTION_PRESENCE_TYPE_OFFLINE) <= 0)
         {
           /* someone is logging off */
-          empathy_sound_manager_play (priv->sound_mgr, window,
+          empathy_sound_manager_play (priv->sound_mgr, NULL,
               EMPATHY_SOUND_CONTACT_DISCONNECTED);
 
           if (g_settings_get_boolean (priv->gsettings_notif,
@@ -1288,7 +1176,7 @@ event_manager_presence_changed_cb (EmpathyContact *contact,
             TP_CONNECTION_PRESENCE_TYPE_OFFLINE) > 0)
         {
           /* someone is logging in */
-          empathy_sound_manager_play (priv->sound_mgr, window,
+          empathy_sound_manager_play (priv->sound_mgr, NULL,
               EMPATHY_SOUND_CONTACT_CONNECTED);
 
           if (g_settings_get_boolean (priv->gsettings_notif,
@@ -1305,7 +1193,6 @@ event_manager_presence_changed_cb (EmpathyContact *contact,
 
 out:
   g_object_unref (presence_mgr);
-  g_object_unref (window);
 }
 
 static GObject *
